package no.miles.mpl.wrapper.datastructures;

import java.io.BufferedOutputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

import com.hp.hpl.jena.ontology.Individual;
import com.hp.hpl.jena.ontology.OntModel;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.rdf.model.ResIterator;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.rdf.model.Statement;

/**
 * Holds a comment (belonging to an issue ++)
 * @author erlingwl
 *
 */
public class Comment {

	public static String DCNS = "http://purl.org/dc/elements/1.1/";
	public static String DCTNS = "http://purl.org/dc/terms/";

	private String author;
	private String content;
	private Date created;

	/**
	 * Constructor which instantiates the current date
	 * @param author
	 * @param content
	 */
	public Comment(String author, String content){
		this.author = author;
		this.content = content;
		this.created = new Date();
	}

	/**
	 * Constructor for an existing comment
	 * @param author
	 * @param content
	 * @param created
	 */
	public Comment(String author, String content, Date created){
		this.author = author;
		this.content = content;
		this.created = created;
	}

	public String toN3(){

		String n3 = null;

		OntModel model = ModelFactory.createOntologyModel();
		model.setNsPrefix("", Issue.BAETLENS);
		model.setNsPrefix("sioc", Issue.SIOCNS);
		model.setNsPrefix("dc", DCNS);
		model.setNsPrefix("dct", DCTNS);
		model.setNsPrefix("foaf", Issue.FOAFNS);

		Resource commentResource = model.createResource(Issue.SIOCNS + "Post");			
		Individual commentIndividual = model.createIndividual(commentResource);


		//xsd:dateTime
		SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-d'T'HH:mm:ss.S");

		if(created != null){			
			String xsdCreated = df.format(created);	
			commentIndividual.addProperty(model.createDatatypeProperty(DCTNS + "created"), xsdCreated);
		}

		if(author != null){			
			Resource authorResource = model.createResource(Issue.SIOCNS + "User");
			Individual authorIndividual = model.createIndividual(authorResource);
			authorIndividual.addProperty(model.createDatatypeProperty(Issue.FOAFNS + "accountName"), author);		
			commentIndividual.addProperty(model.createObjectProperty(Issue.SIOCNS + "has_creator"), authorIndividual);
		}

		if(content != null){
			commentIndividual.addProperty(model.createDatatypeProperty(Issue.SIOCNS + "content"), content);
		}

		//Convert to string
		ByteArrayOutputStream out = new ByteArrayOutputStream();
		BufferedOutputStream bufferedOut = new BufferedOutputStream(out);
		model.write(bufferedOut, "N3");

		n3 = out.toString();

		return n3;

	}

	public static Comment parseN3(String n3) throws Exception{

		Comment comment = null;

		OntModel model = ModelFactory.createOntologyModel();
		ByteArrayInputStream inputStream = new ByteArrayInputStream(n3.getBytes());
		model.read(inputStream, "", "N3");

		Resource commentResource = null;
		ResIterator iterator; 			


		iterator = model.listSubjectsWithProperty(model.createProperty(Issue.RDFNS + "type"), model.createResource(Issue.SIOCNS + "Post"));
		while(iterator.hasNext()){
			commentResource = iterator.nextResource();			
			break;
		}

		if(commentResource != null){


			//xsd:dateTime
			SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-d'T'HH:mm:ss.S");

			//Created
			Date created = null;
			String createdXsd = null;
			Statement createdStatement = commentResource.getProperty(model.createDatatypeProperty(DCTNS + "created"));
			if(createdStatement != null){
				createdXsd = createdStatement.getObject().toString();
			}
			if(createdXsd != null){
				try {
					created = df.parse(createdXsd);
				} catch (ParseException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
					//Ok ??
					System.out.println("Couldn't parse date..");
				}
			}

			//Fetch content
			String content = null;
			Statement contentStatement = commentResource.getProperty(model.createDatatypeProperty(Issue.SIOCNS + "content"));
			if(contentStatement != null){
				content = contentStatement.getObject().toString();	
			}


			//author
			String author = null;
			Statement authorStatement = commentResource.getProperty(model.createObjectProperty(Issue.SIOCNS + "has_creator"));
			if(authorStatement != null){
				Statement authorNameStatement = authorStatement.getResource().getProperty(model.createDatatypeProperty(Issue.FOAFNS + "accountName"));
				if(authorNameStatement != null)
					author = authorNameStatement.getObject().toString();
			}

			if(created != null && author != null && content != null){
				comment = new Comment(author, content, created);
			} else if (author != null && content != null){
				comment = new Comment(author, content);
			} else {
				throw new Exception("Couldn't parse the n3 - author and/or content not found");
			}
		}

		return comment;
	}

	public String getAuthor() {
		return author;
	}

	public void setAuthor(String author) {
		this.author = author;
	}

	public String getContent() {
		return content;
	}

	public void setContent(String content) {
		this.content = content;
	}

	public Date getCreated() {
		return created;
	}

	public void setCreated(Date created) {
		this.created = created;
	}

}
